@@ -6,11 +6,15 @@
  * 1999-02-03 V 1.1.0
  * 1999-06-22 V 1.2.0 beta
  * 1999-09-10 V 1.2.0 final
+ * 2000-03-03 V 1.3.0 beta
+ * 2000-07-15 V 1.3.0 final
+ * 2001-10-10 V 1.3.1  
+ * 2003-07-04 V 1.3.2
  *
  * NOTE: Edit this file with tabstop=4 !
  *
- * Copyright 1996-1999 by Gerhard Buergmann
- * Gerhard.Buergmann@altavista.net
+ * Copyright 1996-2003 by Gerhard Buergmann
+ * gerhard@puon.at
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -36,7 +40,7 @@
 #	include <fcntl.h>
 #endif
 
-#ifdef __MSDOS__
+#if defined(__MSDOS__) && !defined(DJGPP)
 #   include <io.h>
 #	include <dir.h>
 #   define WRITE (O_WRONLY|O_CREAT|O_TRUNC|O_BINARY)
@@ -203,7 +207,7 @@ docmdline(cmdline)
 			sprintf(string, "What?|%s: Not an editor command", cmdbuf);
 			emsg(string);
 			return;
-			}
+		}
 	}
 	cmdname[n] = '\0';
 	if (*cmd == '!') {
@@ -218,7 +222,7 @@ docmdline(cmdline)
 		if (repl_count == -1) {
 			emsg("No previous substitute re|No previous substitute regular expression");
 			return;	/* No prev subst */
-			}
+		}
 		repaint();
 		if (!repl_count) {
 			emsg("Fail|Substitute pattern matching failed");
@@ -293,19 +297,6 @@ docmdline(cmdline)
 		        strcat (oldbuf, p + 1);
 				c_argv[0] = oldbuf;
 		    }
-			/* change '#' to Altname */
-			while ((p = strchr(cmd, '#')) != NULL && *(p - 1) != '\\') {
-				if (altfile == NULL) {
-					emsg("No alternate filename@to substitute for #");
-					return;
-				}
-				*p = '\0';
-				strcpy (cmdbuf, cmd);
-				strcat (cmdbuf, altfile);
-				strcat (cmdbuf, p + 1);
-				strcpy(cmd, cmdbuf);
-				msg(cmd);			/*repeat */
-			}
 		}
 		if (name == NULL && c_argc != 0) name = strdup(c_argv[0]);
 		if (force) {
@@ -324,14 +315,25 @@ docmdline(cmdline)
 						sprintf(string, "File exists@- use \"%s! %s\" to overwrite", cmdname, c_argv[0]);
 						emsg(string);
 						return;
-					} else {
+					} else {		/* APPEND */
+/* We can only append to a regular file! */
+						if (S_ISREG(buf.st_mode)) {
+							if (filemode == PARTIAL) filemode = REGULAR;
+						} else if (S_ISBLK(buf.st_mode)) {
+							emsg("Cannot append to a block special file");
+							return;
+						}
 						ok = save(c_argv[0], start_addr, end_addr, saveflag);
 					}
 				} else {
 					if (saveflag == APPEND) {
 						emsg("No such file");
 						return;
-					} else {
+					} else {		/* WRITE */
+/* If we write the block of a partial file to a new file, it will
+ * become a regular file!
+ */
+						if (filemode == PARTIAL) filemode = REGULAR;
 						ok = save(c_argv[0], start_addr, end_addr, saveflag);
 					}
 				}
@@ -358,7 +360,7 @@ docmdline(cmdline)
 		}
 	} else if (!strncmp("args", cmdname, len) && CMDLNG(4, 2)) {
 		if (chk_comm(NO_ADDR|NO_ARG)) return;
-		strcpy(string, "");
+		string[0] = '\0';
 		for (n = 0; n < numfiles; n++) {
 			if (n == curfile) strcat(string, "[");
 			strcat(string, files[n]);
@@ -387,20 +389,22 @@ docmdline(cmdline)
 		if (c_argc == 0) {
 			sprintf(string, "%c", DELIM);
 			c_argv[0] = string;
-			}
+		}
 		if (chdir(c_argv[0])) sysemsg(c_argv[0]);
 	} else if (!strncmp("edit", cmdname, len) && CMDLNG(4, 1)) {
-		if (chk_comm(NO_ADDR|MAX_ONE_FILE)) return;
-		doecmd(c_argv[0], force);
 	/*
 	 * The command ":e#" gets expanded to something like ":efile", so
 	 * detect that case here.
 	 */
-	} else if (*cmd == 'e' && c_argc == 0) {
-		if (cmd[1] == '!')
-			(void) doecmd(&cmd[2], TRUE);
-		else
-			(void) doecmd(&cmd[1], FALSE);
+		if (*cmd == 'e' && c_argc == 0) {
+			if (cmd[1] == '!')
+				(void) doecmd(&cmd[2], TRUE);
+			else
+				(void) doecmd(&cmd[1], FALSE);
+			return;
+		}
+		if (chk_comm(NO_ADDR|MAX_ONE_FILE)) return;
+		doecmd(c_argv[0], force);
 	} else if (!strncmp("file", cmdname, len) && CMDLNG(4, 1)) {
 		if (chk_comm(NO_ADDR|MAX_ONE_FILE)) return;
 		if (c_argc != 0) name = strdup(c_argv[0]);
@@ -640,24 +644,39 @@ doecmd(arg, force)
 	char	*arg;
 	int		force;
 {
+	char *tmp;
+
+	if (*arg == '\0') arg = NULL;
 	if (!force && edits) {
 		sprintf(string, nowrtmsg, "edit");
 		emsg(string);
+		/*
 		if (altfile)
 			free(altfile);
 		altfile = strdup(arg);
+		*/
 		return FALSE;
 	}
 	if (arg != NULL) {
-		if (name != NULL && strcmp(arg, name) == 0) {
+		if (name != NULL && !strcmp(arg, name)) {
 			if (!edits || (edits && !force))
 				return TRUE;
 		}
-		if (altfile) {
-			free(altfile);
+		if (name != NULL && !strcmp(arg, "#")) {
+			if (altfile == NULL) {
+				emsg("No alternate filename@to substitute for #");
+				return FALSE;
+			}
+			tmp = name;
+			name = altfile;
+			altfile = tmp;
+		} else {
+			if (altfile) {
+				free(altfile);
+			}
+			altfile = name;
+			name = strdup(arg);
 		}
-		altfile = name;
-		name = strdup(arg);
 	}
 	if (name == NULL) {
 		emsg("No file|No current filename");
@@ -709,7 +728,12 @@ sysemsg(s)
 {
 	char	string[256];
 
+#ifdef HAVE_STRERROR
+	sprintf(string, "%s: %s", s, strerror(errno));
+#else
 	sprintf(string, "%s: %s", s, sys_errlist[errno]);
+#endif
+
 	emsg(string);
 }
 
